from pm4pygpu.cases_df import build_cases_df
from pm4pygpu.constants import Constants
import sys


def filter_events(df, min_timest=0, max_timest=100000000000, att=None):
	if att is None:
		col = Constants.TARGET_TIMESTAMP
	else:
		col = Constants.TEMP_COLUMN_1
		df = df.copy()
		df[col] = df[att].astype(int) // 10**6
	df = df.query(col+" >= "+str(min_timest)+" and "+col + " <= "+str(max_timest))
	return df

def filter_cases_contained(df, min_timest=0, max_timest=100000000000, att=None):
	if att is None:
		col1 = Constants.TARGET_TIMESTAMP
		col2 = Constants.TARGET_TIMESTAMP + "_2"
	else:
		col1 = Constants.TEMP_COLUMN_1
		col2 = Constants.TEMP_COLUMN_2
	cdf = build_cases_df(df, att=att)
	cdf = cdf.query(col1+" >= "+str(min_timest)+" and "+col2+" <= "+str(max_timest))[Constants.TARGET_CASE_IDX]
	return df[df[Constants.TARGET_CASE_IDX].isin(cdf)]

def filter_cases_intersecting(df, min_timest=0, max_timest=100000000000, att=None):
	if att is None:
		col1 = Constants.TARGET_TIMESTAMP
		col2 = Constants.TARGET_TIMESTAMP + "_2"
	else:
		col1 = Constants.TEMP_COLUMN_1
		col2 = Constants.TEMP_COLUMN_2
	cdf = build_cases_df(df, att=att)
	query_parts = []
	query_parts.append("(" + col1 + " >= " + str(min_timest)+" and " + col1 + " <= " + str(max_timest) + ")")
	query_parts.append("(" + col2 + " >= " + str(min_timest)+" and " + col2 + " <= " + str(max_timest) + ")")
	query_parts.append("(" + col1 + " <= " + str(min_timest)+" and " + col2 + " >= " + str(max_timest) + ")")
	query = " or ".join(query_parts)
	cdf = cdf.query(query)[Constants.TARGET_CASE_IDX]
	return df[df[Constants.TARGET_CASE_IDX].isin(cdf)]

def timestamp_attribute_values(df, attribute=None, n_values=sys.maxsize):
	if attribute is not None:
		df = df.copy()
		df[Constants.TEMP_COLUMN_1] = df[attribute].astype("int") // 10**6
		attribute = Constants.TEMP_COLUMN_1
	else:
		attribute = Constants.TARGET_TIMESTAMP
	serie = df[attribute].dropna().sort_values()
	lenn = len(serie)
	if n_values < lenn:
		serie = serie.sample(n_values)
	ret = serie.to_arrow().to_pylist()
	for i in range(len(ret)):
		ret[i] = float(ret[i])
	return ret




from pm4pygpu.constants import Constants
from pm4pygpu.start_end_activities import get_end_activities
import sys

def get_variants_df(df):
	return df.groupby(Constants.TARGET_CASE_IDX).agg({Constants.TARGET_ACTIVITY_CODE: "sum", Constants.TARGET_VARIANT_NUMBER: "sum", Constants.TARGET_EV_IDX: "count"}).reset_index()

def filter_on_variants(df, allowed_variants):
	activities = df[Constants.TARGET_ACTIVITY].cat.categories.to_arrow().to_pylist()
	activities = {activities[i]: i for i in range(len(activities))}
	list_tup_vars = []
	for varstri in allowed_variants:
		var = varstri.split(Constants.VARIANTS_SEP)
		v1 = 0
		v2 = 0
		for i in range(len(var)):
			v1 += activities[var[i]]
			v2 += (len(var) + i + 1) * (activities[var[i]] + 1)
		list_tup_vars.append("(" + Constants.TARGET_ACTIVITY_CODE + " == "+str(v1)+" and " +Constants.TARGET_VARIANT_NUMBER+" == "+str(v2) + ")")
	this_query = " or ".join(list_tup_vars)
	cdf = get_variants_df(df)
	cdf = cdf.query(this_query)[Constants.TARGET_CASE_IDX]
	return df[df[Constants.TARGET_CASE_IDX].isin(cdf)]

def get_variants(df, max_des_vars_num=sys.maxsize, return_list=False):
	activities = df[Constants.TARGET_ACTIVITY].cat.categories.to_arrow().to_pylist()
	activities = {activities[i]: i for i in range(len(activities))}
	cdf = get_variants_df(df)
	vars_count = cdf.groupby([Constants.TARGET_ACTIVITY_CODE, Constants.TARGET_VARIANT_NUMBER]).count()[Constants.TARGET_CASE_IDX].to_pandas().to_dict()
	cdf = cdf.groupby([Constants.TARGET_ACTIVITY_CODE, Constants.TARGET_VARIANT_NUMBER]).agg({Constants.TARGET_CASE_IDX: "min", Constants.TARGET_EV_IDX: "count"})
	if max_des_vars_num < sys.maxsize:
		nlarg = min(cdf[Constants.TARGET_EV_IDX].nlargest(max_des_vars_num).to_pandas().to_dict().values())
		cdf = cdf.query(Constants.TARGET_EV_IDX + " >= " + str(nlarg))
	vars0 = df[df[Constants.TARGET_CASE_IDX].isin(cdf[Constants.TARGET_CASE_IDX])][[Constants.TARGET_CASE_IDX, Constants.TARGET_ACTIVITY]].to_arrow().to_pydict()
	cases = {}
	for i in range(len(vars0[Constants.TARGET_ACTIVITY])):
		case = vars0[Constants.TARGET_CASE_IDX][i]
		act = vars0[Constants.TARGET_ACTIVITY][i]
		if not case in cases:
			cases[case] = []
		cases[case].append(act)
	ret = {}
	for case in cases:
		v1 = 0
		v2 = 0
		cases_act = cases[case]
		for i in range(len(cases_act)):
			v1 += activities[cases_act[i]]
			v2 += (len(cases_act) + i + 1) * (activities[cases_act[i]] + 1)
		ret[",".join(cases_act)] = int(vars_count[(v1, v2)])
	if return_list:
		ret = [{"variant": x, "count": y} for x, y in ret.items()]
	return ret



class Constants:
	TARGET_CASE_IDX = "custom_column_case_idx"
	TARGET_ACTIVITY = "custom_column_activity"
	TARGET_ACTIVITY_CODE = "custom_column_activity_code"
	TARGET_TIMESTAMP = "custom_column_timestamp"
	TARGET_EV_IDX = "custom_column_event_idx"
	TARGET_EV_CASE_MULT_ID = "custom_column_ev_case_mult_idx"
	CASE_DURATION = "custom_column_case_duration"

	TARGET_PRE_ACTIVITY = "custom_column_pre_activity_code"
	TARGET_PRE_TIMESTAMP = "custom_column_pre_timestamp"
	TARGET_PRE_CASE = "custom_column_pre_case"
	TARGET_VARIANT_NUMBER = "custom_column_variant_number"
	TIMESTAMP_DIFF = "custom_column_timestamp_diff"

	VARIANTS_SEP = ","
	CUSTOM_PREFIX = "custom_column"
	TEMP_COLUMN_1 = "custom_column_temp_1"
	TEMP_COLUMN_2 = "custom_column_temp_2"



from pm4pygpu.constants import Constants
from pm4pygpu.cases_df import get_first_df, get_last_df


def get_start_activities(df):
	first_df = get_first_df(df)[Constants.TARGET_ACTIVITY]
	sa_dict = first_df.value_counts().to_pandas().to_dict()
	for el in sa_dict:
		sa_dict[el] = int(sa_dict[el])
	return sa_dict

def get_end_activities(df):
	last_df = get_last_df(df)[Constants.TARGET_ACTIVITY]
	ea_dict = last_df.value_counts().to_pandas().to_dict()
	for el in ea_dict:
		ea_dict[el] = int(ea_dict[el])
	return ea_dict

def filter_start_activities(df, list_act, retain=True):
	first_df = get_first_df(df)
	first_df = first_df[first_df[Constants.TARGET_ACTIVITY].isin(list_act)]
	cases = first_df[Constants.TARGET_CASE_IDX].unique()
	if retain:
		return df[df[Constants.TARGET_CASE_IDX].isin(cases)]
	else:
		return df[~df[Constants.TARGET_CASE_IDX].isin(cases)]

def filter_end_activities(df, list_act, retain=True):
	last_df = get_last_df(df)
	last_df = last_df[last_df[Constants.TARGET_ACTIVITY].isin(list_act)]
	cases = last_df[Constants.TARGET_CASE_IDX].unique()
	if retain:
		return df[df[Constants.TARGET_CASE_IDX].isin(cases)]
	else:
		return df[~df[Constants.TARGET_CASE_IDX].isin(cases)]



from pm4pygpu.constants import Constants
from numba import cuda
import numpy as np

def paths_udf(custom_column_temp_1, custom_column_temp_2):
	for i in range(cuda.threadIdx.x, len(custom_column_temp_2), cuda.blockDim.x):
		if i > 0:
			custom_column_temp_1[i] = custom_column_temp_2[i-1]

def get_frequency_dfg(df, att=None):
	if att is None:
		col1 = Constants.TARGET_ACTIVITY
		col3 = Constants.TARGET_PRE_ACTIVITY
		col2 = Constants.TARGET_ACTIVITY_CODE
	else:
		df = df.copy()
		col1 = att
		col2 = Constants.TEMP_COLUMN_2
		col3 = Constants.TEMP_COLUMN_1
		df[col1] = df[col1].astype("category")
		df[col2] = df[col1].cat.codes
		df = df.groupby(Constants.TARGET_CASE_IDX).apply_grouped(paths_udf, incols=[col2], outcols={col3: np.int32})
	av = df[col1].cat.categories.to_arrow().to_pylist()
	av = {i: av[i] for i in range(len(av))}
	dfg = df.query(Constants.TARGET_CASE_IDX+" == "+Constants.TARGET_PRE_CASE).groupby([col3, col2]).count()[Constants.TARGET_VARIANT_NUMBER].to_pandas().to_dict()
	dfg = {(str(av[x[0]]), str(av[x[1]])): int(y) for x, y in dfg.items()}
	return dfg

def get_performance_dfg(df, att=None):
	if att is None:
		col1 = Constants.TARGET_ACTIVITY
		col3 = Constants.TARGET_PRE_ACTIVITY
		col2 = Constants.TARGET_ACTIVITY_CODE
	else:
		df = df.copy()
		col1 = att
		col2 = Constants.TEMP_COLUMN_2
		col3 = Constants.TEMP_COLUMN_1
		df[col1] = df[col1].astype("category")
		df[col2] = df[col1].cat.codes
		df = df.groupby(Constants.TARGET_CASE_IDX).apply_grouped(paths_udf, incols=[col2], outcols={col3: np.int32})
	av = df[col1].cat.categories.to_arrow().to_pylist()
	av = {i: av[i] for i in range(len(av))}
	dfg_perf = df.query(Constants.TARGET_CASE_IDX+" == "+Constants.TARGET_PRE_CASE).groupby([col3, col2]).agg({Constants.TIMESTAMP_DIFF: "mean"})[Constants.TIMESTAMP_DIFF].to_pandas().to_dict()
	dfg_perf = {(str(av[x[0]]), str(av[x[1]])): float(y) for x, y in dfg_perf.items()}
	return dfg_perf

def filter_paths(df0, allowed_paths, att=None, retain=True):
	if att is None:
		col1 = Constants.TARGET_ACTIVITY
		col3 = Constants.TARGET_PRE_ACTIVITY
		col2 = Constants.TARGET_ACTIVITY_CODE
		df = df0
	else:
		df = df0.copy()
		col1 = att
		col2 = Constants.TEMP_COLUMN_2
		col3 = Constants.TEMP_COLUMN_1
		df[col1] = df[col1].astype("category")
		df[col2] = df[col1].cat.codes
		df = df.groupby(Constants.TARGET_CASE_IDX).apply_grouped(paths_udf, incols=[col2], outcols={col3: np.int32})
	av = df[col1].cat.categories.to_arrow().to_pylist()
	av = {av[i]: i for i in range(len(av))}
	allowed_paths = [(av[x[0]], av[x[1]]) for x in allowed_paths]
	filters = []
	for ap in allowed_paths:
		filters.append("(" + col3 + " == " + str(ap[0]) + " and " + col2 + " == " + str(ap[1]) + ")")
	query = " or ".join(filters)
	cdf = df.query(Constants.TARGET_CASE_IDX+" == "+Constants.TARGET_PRE_CASE).query(query)[Constants.TARGET_CASE_IDX]
	if retain:
		return df0[df0[Constants.TARGET_CASE_IDX].isin(cdf)]
	else:
		return df0[~df0[Constants.TARGET_CASE_IDX].isin(cdf)]



from pm4pygpu.constants import Constants
from numba import cuda
import numpy as np

def post_grouping_function(custom_column_activity_code, custom_column_timestamp, custom_column_case_idx, custom_column_pre_activity_code, custom_column_pre_timestamp, custom_column_pre_case, custom_column_variant_number):
	for i in range(cuda.threadIdx.x, len(custom_column_activity_code), cuda.blockDim.x):
		custom_column_variant_number[i] = (len(custom_column_activity_code) + i + 1) * (custom_column_activity_code[i] + 1)
		if i > 0:
			custom_column_pre_activity_code[i] = custom_column_activity_code[i-1]
			custom_column_pre_timestamp[i] = custom_column_timestamp[i-1]
			custom_column_pre_case[i] = custom_column_case_idx[i-1]
		else:
			custom_column_pre_case[i] = -1

def post_filtering(df):
	cdf = df.groupby(Constants.TARGET_CASE_IDX)
	df = cdf.apply_grouped(post_grouping_function, incols=[Constants.TARGET_ACTIVITY_CODE, Constants.TARGET_TIMESTAMP, Constants.TARGET_CASE_IDX], outcols={Constants.TARGET_PRE_ACTIVITY: np.int32, Constants.TARGET_PRE_TIMESTAMP: np.int32, Constants.TARGET_PRE_CASE: np.int32, Constants.TARGET_VARIANT_NUMBER: np.int32})
	df[Constants.TIMESTAMP_DIFF] = df[Constants.TARGET_TIMESTAMP] - df[Constants.TARGET_PRE_TIMESTAMP]
	return df

def prefix_columns(df):
	columns = list(df.columns)
	columns = [x.replace("AAA", ":") for x in columns]
	df.columns = columns
	return df


def apply(df, case_id="case:concept:name", activity_key="concept:name", timestamp_key="time:timestamp"):
	df = prefix_columns(df)
	df[Constants.TARGET_ACTIVITY] = df[activity_key].astype("category")
	df[Constants.TARGET_ACTIVITY_CODE] = df[Constants.TARGET_ACTIVITY].cat.codes
	df[Constants.TARGET_TIMESTAMP] = df[timestamp_key].astype("int") // 10**6
	df[Constants.TARGET_TIMESTAMP + "_2"] = df[Constants.TARGET_TIMESTAMP]
	df[Constants.TARGET_EV_IDX] = df.index.astype("int")
	df = df.sort_values([Constants.TARGET_TIMESTAMP, Constants.TARGET_EV_IDX]).reset_index()
	df[Constants.TARGET_CASE_IDX] = df[case_id].astype("category").cat.codes
	#df = df.sort_values([Constants.TARGET_CASE_IDX, Constants.TARGET_TIMESTAMP, Constants.TARGET_EV_IDX]).reset_index()
	df[Constants.TARGET_EV_IDX] = df.index.astype("int")
	df[Constants.TARGET_EV_IDX] = df[Constants.TARGET_EV_IDX] + 1
	mult_fact = df[Constants.TARGET_EV_IDX].max() + 2
	df[Constants.TARGET_EV_CASE_MULT_ID] = df[Constants.TARGET_CASE_IDX].astype(np.int32) + 1
	df[Constants.TARGET_EV_CASE_MULT_ID] = mult_fact * df[Constants.TARGET_EV_CASE_MULT_ID]
	df[Constants.TARGET_EV_CASE_MULT_ID] = df[Constants.TARGET_EV_CASE_MULT_ID] + df[Constants.TARGET_EV_IDX]
	return post_filtering(df)



from pm4pygpu.constants import Constants
import sys

def get_first_df(df):
	group_df_min = df.groupby(Constants.TARGET_CASE_IDX).agg({Constants.TARGET_EV_CASE_MULT_ID: "min"})
	idxs = group_df_min[Constants.TARGET_EV_CASE_MULT_ID].unique().to_arrow()
	first_df = df[df[Constants.TARGET_EV_CASE_MULT_ID].isin(idxs)]
	return first_df

def get_last_df(df):
	group_df_max = df.groupby(Constants.TARGET_CASE_IDX).agg({Constants.TARGET_EV_CASE_MULT_ID: "max"})
	idxs = group_df_max[Constants.TARGET_EV_CASE_MULT_ID].unique().to_arrow()
	last_df = df[df[Constants.TARGET_EV_CASE_MULT_ID].isin(idxs)]
	return last_df

def build_cases_df(df, att=None):
	if att is None:
		col1 = Constants.TARGET_TIMESTAMP
		col2 = Constants.TARGET_TIMESTAMP + "_2"
	else:
		col1 = Constants.TEMP_COLUMN_1
		col2 = Constants.TEMP_COLUMN_2
		df = df.copy()
		df[col1] = df[att].astype(int) // 10**6
		df[col2] = df[Constants.TEMP_COLUMN_1]
	cases_df = df.groupby(Constants.TARGET_CASE_IDX).agg({col1: "min", col2: "max", Constants.TARGET_EV_IDX: "count"}).reset_index()
	cases_df[Constants.CASE_DURATION] = cases_df[col2] - cases_df[col1]
	cases_df = cases_df.sort_values(col1)
	return cases_df

def filter_on_case_size(df, min_size=1, max_size=1000000000):
	cdf = build_cases_df(df)
	cdf = cdf.query(Constants.TARGET_EV_IDX + " >= "+str(min_size)+" and "+Constants.TARGET_EV_IDX + " <= "+str(max_size))[Constants.TARGET_CASE_IDX]
	return df[df[Constants.TARGET_CASE_IDX].isin(cdf)]

def filter_on_case_perf(df, min_perf=0, max_perf=1000000000):
	cdf = build_cases_df(df)
	cdf = cdf.query(Constants.CASE_DURATION + " >= "+str(min_perf)+" and "+Constants.CASE_DURATION + " <= "+str(max_perf))[Constants.TARGET_CASE_IDX]
	return df[df[Constants.TARGET_CASE_IDX].isin(cdf)]

def get_case_durations(df, n_values=sys.maxsize):
	cdf = build_cases_df(df)
	serie = cdf[Constants.CASE_DURATION].sort_values()
	lenn = len(serie)
	if n_values < lenn:
		serie = serie.sample(n_values)
	ret = serie.to_arrow().to_pylist()
	for i in range(len(ret)):
		ret[i] = float(ret[i])
	return ret

def get_intervals(df, n_values=500):
	cdf = build_cases_df(df)
	lenn = len(cdf)
	if n_values < lenn:
		cdf = cdf.sample(n_values)
	inte = cdf.to_arrow().to_pydict()
	ret = []
	for i in range(len(inte[Constants.TARGET_TIMESTAMP])):
		ret.append((float(inte[Constants.TARGET_TIMESTAMP][i]), float(inte[Constants.TARGET_TIMESTAMP+"_2"][i])))
	return ret

def get_case_size(df):
	ret = build_cases_df(df).groupby(Constants.TARGET_EV_IDX).count().to_pandas().to_dict()[Constants.TARGET_CASE_IDX]
	ret = {int(x): int(y) for x, y in ret.items()}
	return ret

def get_cases_description(df, start_idx=0, end_idx=200, sort_column=Constants.TARGET_TIMESTAMP, ascending=True):
	cdf = build_cases_df(df)
	cdf = cdf.sort_values(sort_column, ascending=ascending)
	desc = cdf.to_arrow().to_pydict()
	ret = []
	for i in range(start_idx, min(end_idx, len(desc[Constants.TARGET_TIMESTAMP]))):
		el = {}
		el["caseDuration"] = float(desc[Constants.CASE_DURATION][i])
		el["startTime"] = float(desc[Constants.TARGET_TIMESTAMP][i])
		el["endTime"] = float(desc[Constants.TARGET_TIMESTAMP + "_2"][i])
		el["caseId"] = str(desc[Constants.TARGET_CASE_IDX][i])
		ret.append(el)
	return ret



from pm4pygpu.constants import Constants
from pm4pygpu.variants import get_variants_df

def num_events(df):
	return int(len(df))

def num_cases(df):
	return int(df[Constants.TARGET_CASE_IDX].nunique())

def num_variants(df):
	var_df = get_variants_df(df)
	var_df = var_df.groupby([Constants.TARGET_ACTIVITY_CODE, Constants.TARGET_VARIANT_NUMBER]).count()
	return int(len(var_df))

def get_events_of_case(df, case_id):
	df2 = df.query(Constants.TARGET_CASE_IDX + " == " + str(case_id))
	df2["@@classifier"] = df2[Constants.TARGET_ACTIVITY]
	cols = list(df2.columns)
	cols = [x for x in cols if not x.startswith("custom_column")]
	df2 = df2[cols]
	ret = df2.to_pandas().to_dict("r")
	return ret

def get_csv(df):
	from io import BytesIO
	f = BytesIO()
	df.to_csv(f)
	return f.getvalue()

def get_xes(df):
	from pm4py.objects.log.log import EventStream
	from pm4py.objects.conversion.log import converter
	from pm4py.objects.log.exporter.xes.exporter import serialize
	cols = [x for x in df.columns if not x == "index"]
	stream = df[cols].to_arrow().to_pydict()
	list_eve = EventStream()
	cols = list(stream.keys())
	for i in range(len(stream[cols[0]])):
		ev = {}
		for k in cols:
			ev[k] = stream[k][i]
		list_eve.append(ev)
	list_eve = converter.apply(list_eve)
	return serialize(list_eve)




from pm4pygpu.constants import Constants
import sys

def get_attributes_list(df):
	attributes = list(df.columns)
	attributes = [x for x in attributes if not x.startswith(Constants.CUSTOM_PREFIX) and not x.startswith("@@")]
	return attributes

def get_attribute_values(df, attribute=Constants.TARGET_ACTIVITY):
	att_dict = df[attribute].value_counts().to_pandas().to_dict()
	for att in att_dict:
		att_dict[att] = int(att_dict[att])
	return att_dict

def attribute_filter_cases(df, list_act, attribute=Constants.TARGET_ACTIVITY, retain=True):
	filt_df = df[df[attribute].isin(list_act)]
	cases = filt_df[Constants.TARGET_CASE_IDX].unique()
	if retain:
		return df[df[Constants.TARGET_CASE_IDX].isin(cases)]
	else:
		return df[~df[Constants.TARGET_CASE_IDX].isin(cases)]

def attribute_filter_events(df, list_act, attribute=Constants.TARGET_ACTIVITY, retain=True):
	if retain:
		return df[df[attribute].isin(list_act)]
	else:
		return df[~df[attribute].isin(list_act)]

def numeric_attribute_filter_cases(df, attribute, val_inf, val_sup):
	filt_df = df.query(str(val_inf) + " <= " + attribute + " <= " + str(val_sup))[Constants.TARGET_CASE_IDX]
	return df[df[Constants.TARGET_CASE_IDX].isin(filt_df)]

def numeric_attribute_filter_events(df, attribute, val_inf, val_sup):
	filt_df = df.query(str(val_inf) + " <= " + attribute + " <= " + str(val_sup))
	return filt_df

def numeric_attribute_values(df, attribute, n_values=sys.maxsize):
	serie = df[attribute].dropna().sort_values()
	lenn = len(serie)
	if n_values < lenn:
		serie = serie.sample(n_values)
	ret = serie.to_arrow().to_pylist()
	for i in range(len(ret)):
		ret[i] = float(ret[i])
	return ret



from pm4pygpu.constants import Constants
from pm4pygpu.cases_df import get_first_df

def sample_events(df, n):
	lenn = len(df)
	ss = min(lenn, n)
	return df.sample(ss)

def sample_cases(df, n):
	cdf = get_first_df(df)
	lenn = len(cdf)
	ss = min(lenn, n)
	sample = cdf.sample(ss)[Constants.TARGET_CASE_IDX].to_arrow()
	return df[df[Constants.TARGET_CASE_IDX].isin(sample)]



from os.path import dirname, join

import pm4pygpu
from setuptools import setup


def read_file(filename):
    with open(join(dirname(__file__), filename)) as f:
        return f.read()


setup(
    name="pm4pygpu",
    version=pm4pygpu.__version__,
    description=pm4pygpu.__doc__.strip(),
    long_description=read_file('README.md'),
    author=pm4pygpu.__author__,
    author_email=pm4pygpu.__author_email__,
    py_modules=[pm4pygpu.__name__],
    include_package_data=True,
    packages=["pm4pygpu"],
    url='http://www.pm4py.org',
    license='GPL-3.0',
    install_requires=[
        "pm4py"
    ]
)
